@@ -1,26 +1,43 @@
 # -*- coding: utf-8 -*-
 '''WebGeoCalc API module.'''
 
+import os
+
 import requests
 
 from .errors import APIError, APIReponseError, KernelSetNotFound, TooManyKernelSets
 from .types import ColumnResult, KernelSetDetails, get_type
-from .vars import API_URL
+from .vars import ESA_URL, JPL_URL
 
-class Api(object):
+
+class Api:
     '''WebGeoCalc API object.
 
     Parameters
     ----------
     url : str, optional
-        API root URL. Default:
+        API root URL.
+        Use ``WGC_URL`` global environment variable if present.
+        If not, fallback on :py:obj:`JPL_URL`:
         ``https://wgc2.jpl.nasa.gov:8443/webgeocalc/api``
 
     '''
 
-    def __init__(self, url=API_URL):
-        self.url = url
+    def __init__(self, url=''):
+        self.url = str(url) if url != '' else os.environ.get('WGC_URL', JPL_URL)
         self._kernel_sets = None
+        self._meta = None
+
+    def __str__(self):
+        return self.url
+
+    def __repr__(self):
+        return f'<{self.__class__.__name__}> {self}'
+
+    def __getitem__(self, key):
+        if key in self.metadata:
+            return self.metadata[key]
+        raise KeyError(key)
 
     def get(self, url):
         '''Generic GET request on the API.
@@ -49,8 +66,8 @@ class Api(object):
         response = requests.get(self.url + url)
         if response.ok:
             return self.read(response.json())
-        else:
-            response.raise_for_status()
+
+        return response.raise_for_status()
 
     def post(self, url, payload):
         '''Generic POST request on the API.
@@ -76,16 +93,17 @@ class Api(object):
         Example
         -------
         >>> API.post('/calculation/new', payload=calculation_payload)  # doctest: +SKIP
-        ('0788aba2-d4e5-4028-9ef1-4867ad5385e0', 'COMPLETE', 0)
+        ('0788aba2-d4e5-4028-9ef1-4867ad5385e0', 'COMPLETE')
 
         '''
         response = requests.post(self.url + url, json=payload)
         if response.ok:
             return self.read(response.json())
-        else:
-            response.raise_for_status()
 
-    def read(self, json):
+        return response.raise_for_status()
+
+    @staticmethod
+    def read(json):
         '''Read content from API JSON reponse.
 
         Parameters
@@ -105,7 +123,7 @@ class Api(object):
 
             If response contents ``result``, then return type will be:
 
-            (`Calculation id`: str, `Phase`: str, `Progress`: int)
+            (`Calculation id`: str, `Phase`: str)
 
             If the response contents ``columns`` and ``rows``, then
             return type will be:
@@ -123,8 +141,11 @@ class Api(object):
             If the format of the API response is unexpected.
 
         '''
+        if 'status' not in json:
+            return json
+
         if json['status'] != 'OK':
-            raise APIError(json['message'])
+            raise APIError(json['error']['shortDescription'])
 
         keys = json.keys()
 
@@ -132,16 +153,18 @@ class Api(object):
             dtype = get_type(json['resultType'])
             return [dtype(item) for item in json['items']]
 
-        elif 'result' in keys:
-            return (json['calculationId'], json['result']['phase'],
-                    json['result']['progress'])
+        if 'result' in keys:
+            phase = json['result']['phase']
+            if phase == 'QUEUED':
+                phase += f' | POSITION: {json["result"]["position"]}'
 
-        elif 'columns' in keys and 'rows' in keys:
+            return json['calculationId'], phase
+
+        if 'columns' in keys and 'rows' in keys:
             cols = [ColumnResult(col) for col in json['columns']]
             return cols, json['rows']
 
-        else:
-            raise APIReponseError(json)
+        raise APIReponseError(json)
 
     def kernel_sets(self):
         '''Get list of all kernel sets available on the webgeocalc server.
@@ -175,10 +198,11 @@ class Api(object):
         kernel_sets = list(filter(lambda x: kernel_set in x, self.kernel_sets()))
         if len(kernel_sets) == 1:
             return kernel_sets[0]
-        elif len(kernel_sets) > 1:
+
+        if len(kernel_sets) > 1:
             raise TooManyKernelSets(kernel_set, kernel_sets)
-        else:
-            raise KernelSetNotFound(kernel_set)
+
+        raise KernelSetNotFound(kernel_set)
 
     def kernel_set_id(self, kernel_set):
         '''Extract kernel set ``id`` based on ``id``, ``name`` or `object`.
@@ -196,13 +220,15 @@ class Api(object):
         '''
         if isinstance(kernel_set, int):
             return kernel_set
-        elif isinstance(kernel_set, str):
+
+        if isinstance(kernel_set, str):
             return int(self.kernel_set(kernel_set))
-        elif isinstance(kernel_set, KernelSetDetails):
+
+        if isinstance(kernel_set, KernelSetDetails):
             return int(kernel_set)
-        else:
-            raise TypeError(f"'kernel_set' must be a 'int', a 'str' of a 'KernelSetDetails' object:\n' + \
-                             '>>> Type({kernel_set}) = {type(kernel_set)}")
+
+        raise TypeError(f"'kernel_set' must be a 'int', a 'str' of a 'KernelSetDetails' object:\n' + \
+                        '>>> Type({kernel_set}) = {type(kernel_set)}")
 
     def bodies(self, kernel_set):
         '''Get list of bodies available in a kernel set.
@@ -273,66 +299,80 @@ class Api(object):
 
         Returns
         -------
-        (str, str, int)
-            Tuple of the calculation status:
-            ``(calculation-id, phase, progress)``
+        (str, str)
+            Tuple of the calculation phase:
+            ``(calculation-id, phase)``
             See: :py:func:`read`.
 
         Example
         -------
         >>> API.calculation_new(calculation_payload)  # doctest: +SKIP
-        ('0788aba2-d4e5-4028-9ef1-4867ad5385e0', 'LOADING_KERNELS', 0)
-
-        Warning
-        ----
-        The ``progress`` attribute is not relevant when ``phase`` is `COMPLETE`.
-
-        In the next release, the API responses may contain ``progress``
-        only when applicable.
+        ('0788aba2-d4e5-4028-9ef1-4867ad5385e0', 'LOADING_KERNELS')
 
         '''
         return self.post('/calculation/new', payload)
 
-    def status_calculation(self, id):
-        '''Gets the status of a calculation.
+    def phase_calculation(self, calculation_id):
+        '''Gets the phase of a calculation.
 
         ``GET: /calculation/{id}``
 
         Parameters
         ----------
-        id: str
+        calculation_id: str
             Calculation id.
 
         Returns
         -------
-        (str, str, int)
-            Tuple of the calculation status:
-            ``(calculation-id, phase, progress)``
+        (str, str)
+            Tuple of the calculation phase:
+            ``(calculation-id, phase)``
             See: :py:func:`read`.
 
         Example
         -------
-        >>> API.status_calculation('0788aba2-d4e5-4028-9ef1-4867ad5385e0')  # noqa: E501  # doctest: +SKIP
-        ('0788aba2-d4e5-4028-9ef1-4867ad5385e0', 'COMPLETE', 0)
-
-        Warning
-        ----
-        The ``progress`` attribute is not relevant when ``phase`` is `COMPLETE`.
-
-        In the next release, the API responses may contain ``progress``
-        only when applicable.
+        >>> API.phase_calculation('0788aba2-d4e5-4028-9ef1-4867ad5385e0')  # noqa: E501  # doctest: +SKIP
+        ('0788aba2-d4e5-4028-9ef1-4867ad5385e0', 'COMPLETE')
 
         '''
-        return self.get(f'/calculation/{id}')
+        return self.get(f'/calculation/{calculation_id}')
 
-    def results_calculation(self, id):
+    def cancel_calculation(self, calculation_id):
+        '''Cancels a previously requested calculation, should this not be completed, or its results..
+
+        ``GET: /calculation/{id}/cancel``
+
+        Calculations that have been already ``DISPATCHED``, previously ``CANCELLED`` or
+        that have ``EXPIRED`` cannot be cancelled, and requesting it will produce an `error`.
+
+        Parameters
+        ----------
+        calculation_id: str
+            Calculation id.
+
+        Returns
+        -------
+        (str, str)
+            Tuple of the calculation phase:
+            ``(calculation-id, phase)``
+            See: :py:func:`read`.
+
+        Example
+        -------
+        >>> API.phase_calculation('0788aba2-d4e5-4028-9ef1-4867ad5385e0')  # noqa: E501  # doctest: +SKIP
+        ('0788aba2-d4e5-4028-9ef1-4867ad5385e0', 'CANCELLED')
+
+        '''
+        return self.get(f'/calculation/{calculation_id}/cancel')
+
+    def results_calculation(self, calculation_id):
         '''Gets the results of a complete calculation.
 
         ``GET: /calculation/{id}/results``
 
         Parameters
         ----------
-        id: str
+        calculation_id: str
             Calculation id.
 
         Returns
@@ -348,8 +388,17 @@ class Api(object):
         ([<ColumnResult> UTC calendar date, ...], [['2000-01-01 00:00:00.000000 UTC', ...], [...]])
 
         '''
-        return self.get(f'/calculation/{id}/results')
+        return self.get(f'/calculation/{calculation_id}/results')
+
+    @property
+    def metadata(self):
+        """API metadata."""
+        if self._meta is None:
+            self._meta = self.get('/')
+        return self._meta
 
 
 # Export default API object
 API = Api()
+JPL_API = Api(JPL_URL)
+ESA_API = Api(ESA_URL)
